# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals


class SimpleProxyField(object):

    """
    Дескриптор, для получения и установки параметров в сессии.
    
    :param key: Ключ.
    :type key: str or unicode
    :param default: Значение по умолчанию. Может быть callable объектом.
    """

    def __init__(self, key, default):
        self.default = default
        self.key = key

    def __get__(self, instance, owner):
        default = self.default() if callable(self.default) else self.default
        return instance.storage.get(self._get_key(instance), default)

    def __set__(self, instance, value):
        instance.storage[self._get_key(instance)] = value

    def _get_key(self, instance):
        return ':'.join((instance.prefix, self.key))


class SessionProxy(object):

    """
    Класс посредник, для удобного получения данных из сессии а также, позволяет 
    не разбрасывать параметры.
    
    :cvar remote_user_id: Идентификатор пользователя с сервера авторизации.
    
    :cvar backend: Наименование бэкенда, по которому пользователь вошел в 
        систему.
    :cvar user_ids_to_select: Кортеж из id пользователей, для возможности 
        предоставления выбора конкретного пользователя на вход.
    :cvar token: Временный токен, используется для проверки запроса после 
        выбора пользователя.
    
    :param request: Запрос.
    :type request: django.http.HttpRequest
    :param prefix: Префикс, для того чтобы избежать конфликтов.
    :type prefix: str or unicode
    """

    remote_user_id = SimpleProxyField('user_id', None)
    backend = SimpleProxyField('backend', None)

    user_ids_to_select = SimpleProxyField('user_ids_to_select', tuple)
    token = SimpleProxyField('token', None)

    def __init__(self, request, prefix='auth'):
        self.storage = request.session
        self.prefix = prefix
